@@ -4,11 +4,13 @@ import unittest
 import logging
 
 from .test_trees import TestTrees
+from .test_tools import TestStandalone
+from .test_reconstructor import TestReconstructor
 
 try:
     from .test_nearley.test_nearley import TestNearley
 except ImportError:
-    pass
+    logging.warn("Warning: Skipping tests for Nearley (js2py required)")
 
 # from .test_selectors import TestSelectors
 # from .test_grammars import TestPythonG, TestConfigG
@@ -18,11 +20,12 @@ from .test_parser import (
         TestEarleyStandard,
         TestCykStandard,
         TestLalrContextual,
-        TestEarleyScanless,
         TestEarleyDynamic,
+        TestLalrCustom,
 
-        TestFullEarleyScanless,
+        # TestFullEarleyStandard,
         TestFullEarleyDynamic,
+        TestFullEarleyDynamic_complete,
 
         TestParsers,
         )
